@@ -7,34 +7,12 @@
 __docformat__ = "restructuredtext en"
 _ = unicode
 
-from os.path import exists
+from os.path import exists, split, join
 from logging import getLogger
 
-from yams import ValidationError
-from logilab.common.modutils import LazyObject
-
 from cubicweb import set_log_methods
 
-VCS_REPOS = {'subversion': LazyObject('cubes.vcsfile.reposvn', 'SVNRepository'),
-             'mercurial': LazyObject('cubes.vcsfile.repohg', 'HGRepository'),
-             }
-
-
-class VCSException(Exception):
-    def __init__(self, repoeid, attr, msgid, msgargs):
-        self.repoeid = repoeid
-        self.attr = attr
-        self.msgid = msgid
-        self.msgargs = msgargs
-
-    def __str__(self):
-        return '%s.%s: %s' % (self.repoeid, self.attr,
-                              self.msgid % self.msgargs)
-
-    def to_validation_error(self, translate):
-        msg = translate(self.msgid) % self.msgargs
-        return ValidationError(self.repoeid, {self.attr: msg})
-
+from cubes.vcsfile import bridge
 
 class VCSRepository(object):
     type = None # to set in concret classes
@@ -51,14 +29,15 @@ class VCSRepository(object):
             fspath = path
         if not exists(fspath):
             msg = _('%s does not exist on the file system')
-            raise VCSException(eid, 'path', msg, path)
+            raise bridge.VCSException(eid, 'path', msg, path)
         try:
             fspath = self.canonicalize(fspath)
         except:
             self.exception('invalid repository')
             msg = _('%s is not a valid %s repository')
-            raise VCSException(eid, 'path', msg, (path, self.type))
+            raise bridge.VCSException(eid, 'path', msg, (path, self.type))
         self.path = fspath
+        self.vf_path_cache = {}
 
     def __str__(self):
         return '%s:%s' % (self.type, self.path)
@@ -76,17 +55,44 @@ class VCSRepository(object):
     def canonicalize(self, path):
         return path
 
-    def import_content(self, source, repoentity):
-        """import content from the repository
-
-        NOTE: server side only method
-        """
+    def import_content(self, repoentity):
+        """import content from the repository"""
         raise NotImplementedError()
 
-    def file_content(self, path, rev):
-        """extract a binary stream with the content of the file at `path` at
-        revision `rev` in the repository
+    def vf_eid(self, session, path, date):
+        """get versioned file eid for the given path, inserting it if necessary
         """
+        try:
+            return self.vf_path_cache[path]
+        except KeyError:
+            directory, fname = split(path)
+            rset = session.unsafe_execute(
+                'Any X WHERE X directory %(dir)s, X name %(name)s, '
+                'X from_repository R, R eid %(r)s',
+                {'dir': directory, 'name': fname, 'r': self.eid})
+            if rset:
+                eid = rset[0][0]
+            else:
+                eid = bridge.import_versioned_file(session, self.eid, date,
+                                                   directory, fname)
+            self.vf_path_cache[path] = eid
+            return eid
+
+    def file_content(self, directory, fname, rev):
+        """extract a binary string with the content of the file at `directory` /
+        `fname` for revision `rev` in the repository
+        """
+        if isinstance(fname, unicode):
+            fname = self.encode(fname)
+        if directory is not None:
+            if isinstance(directory, unicode):
+                directory = self.encode(directory)
+            path = join(directory, fname)
+        else:
+            path = fname
+        return self._file_content(path, rev)
+
+    def _file_content(self, path, rev):
         raise NotImplementedError()
 
     def revision_transaction(self, session, entity):
