@@ -6,32 +6,33 @@
 """
 __docformat__ = "restructuredtext en"
 
-from datetime import datetime
 from os.path import split
+from datetime import datetime
 
 from mercurial import hg, ui, revlog, context, util
 from mercurial.node import nullid, short as short_hex
 
 from cubicweb import Binary, QueryError
-from cubicweb.server.sqlutils import SQL_PREFIX
 
-from cubes.vcsfile.repo import VCSRepository
-from cubes.vcsfile.vcssource import get_vcs_source
+from cubes.vcsfile import repo, bridge
+
+_DELETED = object()
 
 
 def bw_nb_revisions(changelog):
     """return the number of revisions in a repository's changelog,
-    assuring pre mercurial 1.1 compatibility
+    assuring mercurial < 1.1 compatibility
     """
     try:
-        return changelog.count()
-    except AttributeError: # hg >= 1.1
         return len(changelog)
+    except AttributeError: # hg < 1.1
+        return changelog.count()
 
-class HGRepository(VCSRepository):
+
+class HGRepository(repo.VCSRepository):
     type = 'hg'
 
-    def import_content(self, source, repoentity):
+    def import_content(self, repoentity):
         """import content from the mercurial repository"""
         assert self.eid == repoentity.eid
         try:
@@ -44,18 +45,32 @@ class HGRepository(VCSRepository):
         session = repoentity.req
         execute = session.execute
         latestrev = repoentity.latest_known_revision
+        if actualrev < latestrev:
+            # striped repo
+            # very minimal support, this won't work if local number of
+            # revisions has changed due to the strip
+            self.warning('strip detected ? latest known revision is %s, latest '
+                         'repository revision is %s', latestrev, actualrev)
+            execute('DELETE Revision X WHERE X revision >= %(rev)s',
+                    {'rev': actualrev})
+            repoentity.set_attributes(latest_known_revision=actualrev)
+            return
+        self.info('hg repo %s: known revisions up to %s, repo revision %s',
+                  repoentity.path, latestrev, actualrev)
         path_filter = repoentity.subpath
         for i in xrange(latestrev, actualrev):
+            self.info('importing revision %s', i)
             node = changelog.node(i)
-            changeset = short_hex(node)
             ctx = repo.changectx(i)
-            date = datetime.fromtimestamp(ctx.date()[0])#date[0])
-            author = unicode(ctx.user(), self.encoding, 'replace')
-            msg = unicode(ctx.description(), self.encoding, 'replace')
+            date = datetime.fromtimestamp(ctx.date()[0])
             #taglist = ctx.tags() #repo.nodetags(node)
-            fdata = {'author': author, 'date': date, 'description': msg,
-                     'changeset': changeset, 'branch': ctx.branch(),
-                     'revision': i, 'repoeid': self.eid}
+            revdata = {'date': date, 'revision': i,
+                       'author': unicode(ctx.user(), self.encoding, 'replace'),
+                       'description': unicode(ctx.description(), self.encoding, 'replace'),
+                       'changeset': unicode(short_hex(node)),
+                       'branch': unicode(ctx.branch()),
+                       }
+            self.imported_revision = i # for hooks
             parents = [short_hex(n) for n in changelog.parents(node)
                        if n != nullid]
             if parents:
@@ -68,18 +83,16 @@ class HGRepository(VCSRepository):
                                       'X from_repository R, R eid %%(r)s'
                                       % ','.join("'%s'"%cs for cs in parents),
                                       {'r': self.eid})
-            if parents: # retest in case parent revision(s) has not been found
-                fdata['parent_revision'] = [r[0] for r in parents]
+            revdata['parents'] = [r[0] for r in parents]
+            reveid = bridge.import_revision(session, self.eid, **revdata)
             changes = repo.status(ctx.parents()[0].node(), ctx.node())[:5]
             modified, added, removed, deleted, unknown = changes
-            imported = False
             for path in modified + added:
                 upath = unicode(path, self.encoding, 'replace')
                 if not (path_filter is None or upath.startswith(path_filter)):
                     continue
-                fdata['path'] = upath
+                vcdata = {}
                 filectx = ctx[path]
-                fdata['data'] = Binary(filectx.data())
                 renameinfo = filectx.renamed()
                 if renameinfo:
                     oldfile, fileid = renameinfo
@@ -95,37 +108,34 @@ class HGRepository(VCSRepository):
                     if pvc:
                         assert len(pvc) == 1
                         if oldfile in removed:
-                            fdata['vc_rename'] = pvc[0][0]
+                            vcdata['vc_rename'] = pvc[0][0]
                         else:
-                            fdata['vc_copy'] = pvc[0][0]
+                            vcdata['vc_copy'] = pvc[0][0]
                     else:
                         self.error('detected copy or rename of %s@%s but unable'
                                    ' to find associated version content',
                                    oldfile, pcs)
-                source.add_version_content(session, fdata)
-                imported = True
+                bridge.import_version_content(session, self.eid, reveid, upath,
+                                              date, **vcdata)
             for path in removed + deleted:
                 upath = unicode(path, self.encoding, 'replace')
                 if not (path_filter is None or upath.startswith(path_filter)):
                     continue
-                fdata['path'] = upath
-                source.add_deleted_version_content(session, fdata)
-                imported = True
-            if imported:
-                source.revision_imported(session, fdata)
+                bridge.import_deleted_version_content(session, self.eid, reveid,
+                                                      upath, date)
+            bridge.set_at_revision(session, reveid)
         if actualrev > latestrev:
             repoentity.set_attributes(latest_known_revision=actualrev)
 
     def hgrepo(self):
         return hg.repository(ui.ui(), self.path)
 
-    def file_content(self, path, rev):
-        """extract a binary stream with the content of the file at `path` at
+    def _file_content(self, path, rev):
+        """extract a binary string with the content of the file at `path` for
         revision `rev` in the repository
         """
-        assert isinstance(path, str)
         ctx = self.hgrepo().changectx(rev)
-        return Binary(ctx[path].data())
+        return ctx[path].data()
 
     def revision_transaction(self, session, entity):
         """open a transaction to create a new revision corresponding to the
@@ -133,20 +143,20 @@ class HGRepository(VCSRepository):
         """
         return HGTransaction(self, session, entity)
 
-    def add_versioned_file_content(self, session, transaction, vf, entity, data):
+    def add_versioned_file_content(self, session, transaction, vf, entity,
+                                   data):
         """add a new revision of a versioned file"""
         vfpath = self.encode(vf.path)
-        transaction.changes[vfpath] = context.memfilectx(vfpath, data.getvalue(),
-                                                         islink=False, isexec=False,
-                                                         copied=None)
+        transaction.changes[vfpath] = context.memfilectx(
+            vfpath, data.getvalue(), islink=False, isexec=False, copied=None)
         return vf.path
 
-    def add_versioned_file_deleted_content(self, session, transaction, vf, entity):
+    def add_versioned_file_deleted_content(self, session, transaction, vf,
+                                           entity):
         """add a new revision of a just deleted versioned file"""
         transaction.changes[self.encode(vf.path)] = _DELETED
         return vf.path
 
-_DELETED = object()
 
 class HGTransaction(object):
     def __init__(self, repohdlr, session, revision):
@@ -181,7 +191,7 @@ class HGTransaction(object):
     def commit(self):
         if not self.changes:
             raise QueryError('nothing changed')
-        # XXX merging, new branches
+        # XXX merging branches
         encode = self.repohdlr.encode
         branch = encode(self.revision.get('branch') or u'default')
         if branch != u'default':
@@ -203,12 +213,17 @@ class HGTransaction(object):
         node = self.repo._commitctx(ctx, force=True, force_editor=False,
                                     empty_ok=True, use_dirstate=False,
                                     update_dirstate=False)
+        # update revision's record to set correct changeset
+        #
         # take care, self.repo != self.session.repo (eg mercurial repo instance
         # / rql repository)
-        source = get_vcs_source(self.session.repo)
-        source.local_update_entity(self.session, self.revision,
-                                   {SQL_PREFIX + 'eid': self.revision.eid,
-                                    SQL_PREFIX + 'changeset': unicode(short_hex(node))})
+        source = self.session.repo.system_source
+         # remove previous cached value to only save the changeset
+        self.revision.clear()
+        self.revision['eid'] = self.revision.eid
+        self.revision['changeset'] = unicode(short_hex(node))
+        source.update_entity(self.session, self.revision)
+        # XXX restore entity's dict?
         self.rollback()
 
     def rollback(self):
