@@ -89,9 +89,9 @@ class DeletedVersionContent(EntityType):
 
 class VersionContent(DeletedVersionContent):
     """actual content of a versioned file, for a given revision"""
-
-    data = Bytes(required=True, fulltextindexed=True,
-                 description=_("file's data"))
+    # not required to avoid yams validation error since it's not actually stored
+    # in the database
+    data = Bytes(fulltextindexed=True, description=_("file's data"))
     data_format = String(required=True, meta=True, maxsize=50,
                          description=_('MIME type of the file.'))
     data_encoding = String(meta=True, maxsize=20,
@@ -102,16 +102,14 @@ class VersionContent(DeletedVersionContent):
 
 
 class from_repository(RelationType):
-    # NOTE: should't be inlined since Repository entities are stored outside the
-    # vcs source
-    pass
+    inlined = True
 
 class content_for(RelationType):
     subject = ('DeletedVersionContent', 'VersionContent')
     object = 'VersionedFile'
     composite = 'object'
     cardinality = '1*'
-    inlined = True # how ?
+    inlined = True
 
 class from_revision(RelationType):
     inlined = True
