@@ -144,7 +144,8 @@ class VFPutUpdateAction(VFAction):
 
 class VFRmAction(VFAction):
     id = 'vcsrmaction'
-    __select__ = VFAction.__select__ & score_entity(lambda x: not x .revision_deleted())
+    __select__ = VFAction.__select__ & \
+                 score_entity(lambda x: not x.revision_deleted())
 
     title = _('mark this file as deleted in the repository')
     category = 'moreactions'
