@@ -20,6 +20,8 @@ from cubicweb.web import (Redirect, controller, form,
                           formfields as fields, formwidgets as fwdgs)
 from cubicweb.web.views import autoform, editforms
 
+from cubes.vcsfile import IMMUTABLE_ATTRIBUTES
+
 
 class RevisionCreationForm(autoform.AutomaticEntityForm):
     # vfeid needed for the inline-creation of VersionContent
@@ -61,15 +63,32 @@ class VCInlinedCreationFormView(editforms.InlineEntityCreationFormView):
         form.form_add_hidden('content_for', self.orig_entity.file.eid,
                              eidparam=True)
 
+def filter_out_immutable_rels(func):
+    def wrapper(self, *args, **kwargs):
+        entity = self.edited_entity
+        if entity.has_eid():
+            immutable = IMMUTABLE_ATTRIBUTES
+        else:
+            immutable = ('Revision.revision',)
+        for rschema, tschemas, role in func(self, *args, **kwargs):
+            if role == 'subject':
+                etype = entity.id
+            else:
+                etype = tschemas[0]
+            if '%s.%s' % (etype, rschema) in immutable:
+                continue
+            yield rschema, tschemas, role
+    return wrapper
 
 class VCRevisionEditionForm(autoform.AutomaticEntityForm):
-    # DeletedVersionContent has nothing editable
-    __select__ = implements('VersionContent', 'Revision')
-    attrcategories = ('primary', 'secondary', 'generated')
-    EDITABLE_ATTRIBUTES = ('description', 'author', 'data_format', 'data_encoding')
-    def editable_attributes(self):
-        return [(rschema, x) for rschema, x in super(VCRevisionEditionForm, self).editable_attributes()
-                if rschema.type in self.EDITABLE_ATTRIBUTES]
+    __select__ = implements('Revision', 'VersionedFile',
+                            'VersionContent', 'DeletedVersionContent')
+    relations_by_category = filter_out_immutable_rels(
+        autoform.AutomaticEntityForm.relations_by_category)
+    srelations_by_category = filter_out_immutable_rels(
+        autoform.AutomaticEntityForm.srelations_by_category)
+    inlined_relations = filter_out_immutable_rels(
+        autoform.AutomaticEntityForm.inlined_relations)
 
 
 class VFUploadFormView(form.FormViewMixIn, EntityView):
