@@ -20,11 +20,9 @@ def render_entity_metadata(self, entity):
     entity.view('metadata', w=self.w)
     if entity.description:
         self.w(tags.div(entity.description, klass='summary'))
-    # XXX unless there is a standard way to pass the (mandatory) branch
-    #     this breaks on hg repos
-    #if not entity.is_head():
-    #    msg = self.req._('this file has newer revisions')
-    #    self.w(tags.div(msg, klass='warning'))
+    if not entity.is_head(entity.rev.branch):
+        msg = self.req._('this file has newer revisions')
+        self.w(tags.div(msg, klass='warning'))
 
 for etype in ('DeletedVersionContent', 'VersionContent'):
     uicfg.primaryview_section.tag_attribute((etype, 'revision'), 'hidden')
@@ -121,19 +119,25 @@ class RevisionPrimaryView(PrimaryView):
     def render_entity_attributes(self, rev):
         self.field('description', html_escape(rev.description))
         self.field('author', html_escape(rev.author))
+        if rev.branch:
+            self.field('branch', html_escape(rev.branch))
         self.field('from_repository', rev.repository.dc_title())
         parents = rev.related('parent_revision')
         if parents:
             self.field('parent_revision', self.view('csv', parents))
+        children = rev.related('parent_revision', 'object')
+        if children:
+            self.field('parent_revision_object', self.view('csv', children))
 
     def render_entity_relations(self, rev):
-        _ = self.req._
-        self.w(u'<h2>%s</h2>' % _('Objects concerned by this revision'))
         versioned = rev.reverse_from_revision
         if versioned:
             self.w(u'<table class="listing">')
-            self.w(u'<tr><th>%s</th></tr>' % _('objects concerned by this revision'))
+            self.w(u'<tr><th>%s</th></tr>' %
+                   self.req._('objects concerned by this revision'))
             for obj in versioned:
                 self.w(u'<tr><td><a href="%s">%s</a></td></tr>' % (
                     obj.absolute_url(), html_escape(obj.dc_title())))
             self.w(u'</table>')
+        else:
+            self.w(u'<p>%s</p>' % self.req._('this revision hasn\'t modified any files'))
